;;; dispfonts.el --- Display Fonts
;;;
;;; Version: 2021/03/30
;;;
;;; Install
;;;  copy dispfonts.el to /usr/share/emacs/site-lisp/.
;;;
;;; Setting
;;;  add to .emacs
;;;  ----
;;;  (require 'dispfonts)
;;;  ----
;;;
;;; Start
;;;  M-x dispfonts
;;;

;;; Code:
(defcustom dispfonts-sample-string "今日は良い天気ですね。     The weather is wonderful today!     0123456789"
  "dispfonts sample string" :group 'dispfonts :type 'string)
(defcustom dispfonts-font-height    1.0
  "dispfonts font height"   :group 'dispfonts :type 'number)
(defcustom dispfonts-tab-width      35
  "dispfonts tab width"     :group 'dispfonts :type 'number)

(defvar dispfonts-buffername "*dispfonts*"
  "Display fonts")

(defvar dispfonts-mode-line-format
  '("    "
    mode-line-frame-identification
    mode-line-buffer-identification
    "(%l,%c)"))

(defvar dispfonts-local-map nil)

(defun dispfonts (&optional othwin)
  "Display fonts"
  (interactive)

  (if (not (window-system))
      (message "Not valid for no-window-system")
    (let ((selected))
      (setq selected (selected-window))
      (if othwin
          (switch-to-buffer-other-window dispfonts-buffername)
        (switch-to-buffer dispfonts-buffername))
      (setq major-mode 'dispfonts-mode
            mode-name "dispfonts mode")
      (setq dispfonts-local-map (make-keymap))
      (define-key dispfonts-local-map "q"  'dispfonts-close)
      (use-local-map dispfonts-local-map)
      (select-window selected)
      (dispfonts-redraw-buffer))
    ))

(defun dispfonts-close ()
  "Close dispfonts buffer"
  (interactive)
  (kill-buffer dispfonts-buffername))

(defun dispfonts-redraw-buffer ()
  (with-current-buffer (get-buffer-create dispfonts-buffername)
    (setq truncate-lines t)
    (setq cursor-type t)
    (setq tab-width dispfonts-tab-width)
    (read-only-mode 0)
    (buffer-disable-undo)
    (erase-buffer)

    (let ((fontlist '()))
      (dolist (f (delete-dups (font-family-list)))
        (add-to-list 'fontlist (decode-coding-string f 'utf-8)))
      (setq fontlist (sort fontlist 'string<))
      (dolist (font fontlist)
        (insert
         (format "%s\t%s\n"
                 font
                 (propertize dispfonts-sample-string
                             'face (list :family font :height dispfonts-font-height)))))
      )
    (goto-char (point-min))
    (setq mode-line-format dispfonts-mode-line-format)
    (read-only-mode 1)
    ))

(provide 'dispfonts)
